function [erc,j] = ERC(Phi,S,mode)
% function [erc,j] = ERC(Phi,S,mode)
%==========================================================================
%  FILE:           ERC.m
%  AUTHOR:         Andreas Tillmann, TU Braunschweig, Germany                                         
%==========================================================================
%                                                                         
%  DESCRIPTION:    This function computes the exact recovery coefficient  
%                  (ERC; coined by J.Tropp) for a given matrix Phi and    
%                  support (column index) set S, i.e.,
%                       ERC = max_{j\notin S} || pinv(Phi_S)a_j ||_1,
%                  where pinv(Phi_S) denotes the pseudo-inverse of the re-
%                  striction of Phi to the columns indexed by S. 
%                  If ERC(Phi,S)<1, any vector x with support(x)=S can be 
%                  recovered by Basis Pursuit, i.e., x is the unique solu-
%                  tion to the L1-norm minimization problem 
%                       min ||y||_1 s.t. Ay = b:=Ax.       
%                                                                         
%  INPUT:   Phi    The matrix to be considered                    
%           S      The index set corresponding to vector support  
%           mode   A string argument specifying how the computations are  
%                  carried out; feasible values are     
%                  'lins' -> use linesolve, or CG if Phi is sparse
%                  'cg'   -> use CG (independent of sparsity of Phi)
%                  'pinv' -> use MATLAB's pinv-function or a modified ver-
%                            sion that can handle sparse matrix input 
%                            (output is dense)  
%                  [ optional; default is mode = 'pinv' ]             
%  OUTPUT:  erc    The exact recovery coefficient ERC(Phi,S)      
%           j      The column index (from the complement of S) which    
%                  defines the maximum in the ERC definition                                                                          
%==========================================================================
% $Revision$
% $Author$
% $Date$
%==========================================================================
if( nargin < 3 )
    mode = 'pinv'; % default: use pinv or its modified variant, see below
end
% Initialize ERC(Phi,S):
erc = 0;

% Remaining column indices (complement of support set S):
SC = 1:size(Phi,2);
SC(S) = [];

% Compute pseudoinverse of PhiS: PsInv = inv(Phi(S)'*Phi(S))*Phi(S)'
if( strcmp(mode,'lins') && ( ~issparse(Phi) )) 
    % use linsolve
    PsInv = linsolve(Phi(:,S)'*Phi(:,S),Phi(:,S)');
elseif( strcmp(mode,'cg') || (strcmp(mode,'lins') && ( issparse(Phi) )) )
    % use CG
    [m k] = size(Phi(:,S));
    PsInv = zeros(k,m); % preallocate space 
    for j = 1:m
        %[PsInv(:,j),~] = pcg(Phi(:,S)'*Phi(:,S),Phi(j,S)');
        PsInv(:,j) = CG(Phi(:,S)',[],Phi(j,S)',1e-6,min(k,20),zeros(k,1));
    end
elseif( strcmp(mode,'pinv') ) % use pinv (or pinvSp, resp.)
    PsInv = pinvSp(Phi(:,S));    
else
    disp('Unknown mode. Choose \''lins\'',\''cg\'' or \''pinv\''.\n');
    return;
end

% Compute ERC = max{ ||PsInv*Phi(:,k)||_1 : k in SC }:
for k = SC
    tmp = norm( PsInv*Phi(:,k), 1);
    if( tmp > erc )
        j = k;
        erc = tmp;
    end
end
