function [ind,w] = HSEext(x,A,Sx,h,mode)
% function [ind,w] = HSEext(x,A,Sx,h,mode)
% =========================================================================                                                                         
%  FILE:           HSEext.m  % HSE = Heuristic Support Evaluation, extended   
%  AUTHOR:         Andreas M. Tillmann, TU Braunschweig, Germany                  
% =========================================================================
%  DESCRIPTION:    This function (heuristically) evualuates the optimality 
%                  condition of the L1-minimization problem, i.e., that 
%                  the subdifferential contains a subgradient h such that 
%                  P*h=0, where P denotes the projection matrix (P*h is the 
%                  subgradient, projected onto the nullspace of A, where A 
%                  stems from the constraint system A*x=b).
%                  To avoid the need for an explicit matrix P, the equiva-
%                  lent condition that there is a vector w of length m 
%                  (row dim. of A) such that (-)A'*w is in the subdifferen-
%                  tial at the point x, is checked instead. The optimality
%                  criterion can hence be formulated as the problem of 
%                  finding a vector w such that
%                  A(:,Sx)^T * w = -sign(x(Sx)) 
%                           ( on the (approx.) support Sx of x ),
%                  -1 <= (A^T * w)_i <= 1    for all i=1,...,n
%
%                  This function is only a heuristic because it ignores the
%                  second constraint when computing w, which is done simply
%                  by calculating an approximate solution to the normal 
%                  equation A(:,Sx)*A(:,Sx)^T *w = -A(:,Sx)*sign(x(Sx))
%                  induced by the first (sign-)constraint. Note that in 
%                  particular, HSE does not check overall existence of a w
%                  satisfying the above constraints, but looks for a speci-
%                  fic vector w; see Remark below.
%                  Hence, this function may return a WRONG indication for 
%                  optimality, and may also suggest optimality if by 
%                  approximating the support a part of the true optimal 
%                  support was cut off (for a subset of the support, there 
%                  also is A'w in the subdifferential, having values +/-1 
%                  on "off-support"-entries). 
%
%                  Remark: Empirically, the HSE was found to work well 
%                  when the solution x is the unique l1-minimizing solution
%                  to Ax=b and there are no off-support entries equal to 
%                  +/-1 in the subgradient A'w computed here. Otherwise, 
%                  HSE may fail to recognize optimality, since in that 
%                  case, the normal equation approach used here will not 
%                  necessarily work (see Fuchs (2004)).         
%                  The extension (compared to HSE.m) here performs an addi-
%                  tional check in case the regular HSE fails, based on the
%                  possibility that the specific w computed by the HSE does
%                  not yield A'w in the subgradient at x, but that there 
%                  nevertheless does exist such a w and that the support of
%                  an optimal x has indeed been correctly approximated to
%                  begin with. For details see the description of the addi-
%                  tional (optional) input 'mode' below.
%
%  INPUT:          x    The vector whose subdifferential is to be evaluated
%                  A    The coefficient matrix of the constraint Ax=b
%                  Sx   Approx. support of vector x (index vector)
%                       [ optional; if argument Sx is not passed, then Sx
%                         will be supp(x,1e-9) by default ]
%                  h    A subgradient of ||.||_1 at x
%                       [ optional; default is h = sign(x) ]
%                  mode Can be either empty to perform HSE as described a-
%                       bove, or contain the right hand side vector b, to
%                       perform an additional check, if HSE does not reco-
%                       nize optimality: Compute x_Sw=A(:,Sw)\b for Sw the
%                       set of indices where abs(A'*w) >= 1, check feasibi-
%                       lity of x_Sw and, if feasible, re-approximate the
%                       support as Swx (using the default hard threshold),
%                       and recompute x_Swx. If the thus obtained x is fea-
%                       sible, has a smaller objective value (l1-norm) than
%                       the input x, and Sx = Swx, x is considered optimal 
%                       by this function.  
%                       The use of this modification of HSE.m is inteded to
%                       overcome not recognizing the optimal support in ca-
%                       ses where the normal equation approach of HSE fails
%                       and is suggested to be employed algorithmically 
%                       only if the approx. support remained the same for a 
%                       not too small number of subsequent iterations.
%                       [ optional; default is mode = [] ]
%
%  OUTPUT:         ind  Optimality indicator, a value of 1 indicates
%                       that the provided x is an optimal solution of
%                       min ||x||_1 s.t. Ax=b
%                  w    The dual variable s.t. A'w hopefully is a subgrad.
%                       of ||.||_1 at x, scaled to be dual-feasible so it
%                       can be used for computation of a valid dual lower
%                       bound -b'w<=min ||x||_1 s.t. Ax=b
% =========================================================================
% $Revision: 121 $
% $Author: andtillm $
% $Date: 2011-04-26 14:54:45 +0200 (Di, 26. Apr 2011) $
% =========================================================================
if( nargin < 3 )
    Sx = supp(x,1e-9);
end
if( nargin < 4 )
    h = sign(x);
end
if( nargin < 5 )
    mode = [];
end

% calculate approx. solution to A[sx]A[sx]'w = A[sx]sign(x(sx)) :
ASx = A(:,Sx);
%w = CG( ASx, [], -ASx*h(Sx), 1e-9, 20, zeros(size(A,1),1) );
w = ASx*CG(ASx', [], -h(Sx), 1e-9, 20, zeros(numel(Sx),1) );

% hoping that, in an (approx.) optimal x, the solution w which satisfies 
% the constraint "A'*w in [-1,1]" (A'*w in subdiff. to l1-norm; ignored above) 
% was actually found ... :
ATw = A'*w;
temp = norm(ATw,'inf');
% unless b=0 in the original problem, x_opt will always have at least one
% nonzero entry, hence any subgradient to an optimal point must have at 
% least one entry equal to +/- 1 !

if( (temp < 1+1e-6) && (temp > 1-1e-6) ) % allow errors/infeas. up to 1e-6
    ind = 1; % optimality condition (approximately) satisfied
else
    ind = 0; % very likely not optimal 
end

% extension to HSE: additional check performed if regular HSE fails;
% requires argument mode was passed with value b (right hand side of
% constraint system Ax=b).
if( (ind == 0) && (~isempty(mode)) )
    Sw = find(abs(ATw)>1-1e-6);
    if( numel(Sw) <= size(A,2) ) % do not continue, if |Sw|>m
        xSw = zeros(size(x)); xSw(Sw) = A(:,Sw)\mode;
        if( norm(A*xSw-mode) < 1e-6 )
            Swx = supp(xSw,1e-9);
            if( all(Swx == Sx) )       
                xSw = zeros(size(x)); xSw(Swx) = A(:,Swx)\mode;
                if( (norm(A*xSw-mode) < 1e-6) && (norm(xSw,1)<norm(x,1)+1e-6) )
                    ind = 1;
                end
            end
        end
    end    
end

if( nargout > 1 )
    w = w./temp; % ensuring w is a feasible dual solution, even if it does 
end              % not yield a subgradient A'w of ||.||_1 at x