function [y,success,w] = source_element(A,x,varargin)
%function [y,success,w] = source_element(A,x)
% The function calculates a so called source element for the
% l^1-minimization problem
%  min ||x||_1 s.t. Ax=b.
% The is for a k by n matrix A and an n-vector x a vector y, such that A'*y
% is in the subgradient of x. If such a vector y does not exist, the
% routine reports failure. The method first find a vector w which is in the
% intersection of the range of A' and the subgradient of x. This is done by
% the 'projection onto convex sets' method, i.e. it alternatingly
% projecting onto the range of A' and the subgradient of x. Then the
% equation A'*y = w is solved.
%
% Input:
%  A: A k by n matrix.
%  x: An n-vector.
%
% Optional input:
%  maxIter: Maximal number of iteration for solving the constraint
%     minimization problem (default: 10*n).
%  tol: Tolerance of the residual in the algorithm for the solution of
%     the constraint minimization problem (default: 1e-6).
%  Verbose: no output on screen (0, default), lots of output (1).
%  use_cg: Use the cg-method to calculate the projection onto the range of
%     A' (default: 1 for sparse A, 0 else).
%
% Output:
%   A k-vector y such that A'*y is an element of the multivalued sign of x,
%   and (optionally) an indicator for success (1 if such a y was found) and
%   the subgradient w=A'*y itself


% $Revision$
% $Author$
% $Date$



maxIter = [];
tol = 1e-6;
verbose = 0;
use_cg = 0;
if (rem(length(varargin),2)==1)
  error('Optional parameters should always go by pairs');
else
  for i=1:2:(length(varargin)-1)
    switch lower(varargin{i})
     case 'maxiter'
       maxIter = varargin{i+1};
     case 'verbose'
       verbose = varargin{i+1};
     case 'tol'
       tol = varargin{i+1};
     case 'use_cg'
       use_cg = varargin{i+1};
     otherwise
      % Hmmm, something wrong with the parameter string
      error(['Unrecognized option: ''' varargin{i} '''']);
    end;
  end;
end

n = size(A,2);
if length(x)~=n
    error('Dimension mismatch: Number of colums of the matrix should match the length of the vector')
end
if isempty(maxIter)
    maxIter = 10*n;
end
maxIter_cg = round(n/2);
lambda = zeros(size(A,1),1); % initialize lambda as starting point for own CG funtion

if issparse(A) || use_cg
    %AAT = @(y) A*(A'*y); % needed only when using MATLABs pcg function
    w0 = zeros(n,1);
    w0(x>0) = +1;
    w0(x<0) = -1;
    w = w0;
    for iter=1:maxIter
        wold = w;
        %[lambda,flag,relres,cgiter] = pcg(AAT,A*wold,n*tol/iter^2,maxIter_cg); % MATLABs own CG
        %lambda = CG(A,[],A*wold,n*tol/iter^2,maxIter_cg,lambda); % own CG, use lambda as starting point
        % How to choose the tolerance for cg? Seems to depend on the size
        % of A... Probably decrease through iteration to speed up?

        % alternative use of CG:
        lambda = CG(A,[],A*wold,1e-6,maxIter_cg,lambda); 
        % Possibly: kind of adjusts itself, since the closer the starting
        % point is to the solution, the faster CG should converge        
                                                            
        %if flag~=0  % needed only when using MATLABs pcg function
        %    error('A problem with the cg-solver occured. Could not project accurately enough onto rg A^T.')
        %end
        Pwold = A'*lambda;
        error_rangeAt = norm(Pwold-wold);
        w = Ps(Pwold,w0);
        error_subgrad = norm(w - Pwold);
        if verbose
            fprintf('% 6d | pocs: dist to range: %4.2e, dist to subgrad: %4.2e\n',iter,error_rangeAt,error_subgrad)
        end
        if max(error_rangeAt,error_subgrad) < tol
            if verbose
                fprintf('pocs converged after %d iterations...\n',iter)
            end
            break
        end
    end
    
    y = lambda;
else
    U = orth(A');
    w0 = zeros(n,1);
    w0(x>0) = +1;
    w0(x<0) = -1;
    w = w0;
    for iter=1:maxIter
        wold = w;
        Pwold = U*(U'*wold); % Projection onto rg A'
        error_rangeAt = norm(Pwold-wold);
        w = Ps(Pwold,w0);
        error_subgrad = norm(w - Pwold);
        if verbose
            fprintf('% 6d | pocs: dist to range: %4.2e, dist to subgrad: %4.2e\n',iter,error_rangeAt,error_subgrad)
        end
        if max(error_rangeAt,error_subgrad) < tol
            if verbose
                fprintf('pocs converged after %d iterations...\n',iter)
            end
            break
        end
    end
    y = A'\w;
end
if verbose && max(error_rangeAt,error_subgrad) >= tol
    %error('Range of A^T and the subgradient of x have empty intersection. Try using a more sparse x.')    
    fprintf('Range of A^T and the subgradient of x have empty intersection. Try using a more sparse x.')    
end

if(nargout > 1)
    success = max(error_rangeAt,error_subgrad) < tol;
end

function y = C(x,lambda)
% Cut-off function
y = sign(x).*min(abs(x),lambda);


function y = Ps(x,pattern)
% 
y = 0*x;
y(pattern > 0) = 1;
y(pattern < 0) = -1;
y(pattern== 0) = C(x(pattern==0),1);